/********************************************************
 * Data Visualization - 2D Matrix Visualization         *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Hilda Chang HE                               *
 * Date: January. 24. 2016                              *
 *                                                      *
 * Purpose: Howework1b for MAT259A                      *
 *          2D Matrix data visualization.               *
 ********************************************************/

//variables declaration
int years = 10;
int cellWidth; // = (int)max(width/17, height/9.6);//60;
int centerMargin = 60;
int upperMargin = 150;

Table table;
int row, col;
float[][] ficFrequency;
float[][] nonficFrequency;
float[] ficDays;
float[] nonficDays;
float minF, maxF;
float minD, maxD;

float[][] diameters;
float[][] xPositions;
float[][] yPositions;

int colorMode = 1;
PFont hind;
PFont hindB;

float[] demoPositionX;
float[] demoDiameter;
float space;

boolean numDisplay;

void setup() {
  //setup window
  size(1024, 740);
  cellWidth = (int)min(width/17, height/12);//60;
  surface.setResizable(true);
  //load data
  table = loadTable("data_2DArray.csv", "header");
  row = table.getRowCount(); 
  col = table.getColumnCount();
  ficFrequency = new float[years][years];
  nonficFrequency = new float[years][years];
  ficDays = new float[years];
  nonficDays = new float[years];
  for(int i=0; i<row; i+=2) {
    for (int j=3; j<col; j++) {
      ficFrequency[i/2][j-3] = table.getFloat(i, j);// did normalization
      //println(table.getFloat(i, j));
    }
  }
  for(int i=1; i< row; i+=2) {
    for (int j=3; j< col; j++) {
      nonficFrequency[(i-1)/2][j-3] = table.getFloat(i, j);// did normalization
    }
  }
  for(int i = 0; i < row; i+=2){
    ficDays[i/2] = table.getFloat(i, 2);
    nonficDays[i/2] = table.getFloat(i+1, 2);
  }
  minF = min(min2D(ficFrequency), min2D(nonficFrequency));
  maxF = max(max2D(ficFrequency), max2D(nonficFrequency));
  minD = min(min(ficDays), min(nonficDays));
  maxD = max(max(ficDays), max(nonficDays));
  
  println("let's test data " + ficFrequency[0][8]);
  println("let's test data " + nonficFrequency[0][8]);
  println("let's test data " + ficDays[9]);
  println("let's test data " + nonficDays[9]);
  
  //initiate color bar
  colorBarL = loadImage(colorPathL[colorMode-1]);
  colorBarR = loadImage(colorPathR[colorMode-1]);
  
  //load font
  hind = createFont("Hind-Regular.ttf", 32);
  hindB = createFont("Hind-Semibold.ttf", 32);
  
  //size bar initialiaztion
  demoPositionX = new float[years-1];
  demoDiameter = new float[years-1];
  
  //number display initialization
  numDisplay = false;
}

void draw() {
  background(240); 
  
  //calculate circle's diameter
  diameters = new float[years][(years+1)*2];
  int dummy = 10;
  for(int j = 0 ; j < (years+1)*2; j+=2){
    for(int i = 0; i < dummy; i++){
      if(j ==0 && i ==0){
        diameters[i][j] = 0;
        diameters[i][j+1] = 0;
      }
      else{
        //other cell
        //diameters[i][j]= nonficFrequency[i][j/2+i-1];
        //diameters[i][j+1]= ficFrequency[i][j/2+i-1];
        diameters[i][j]= map(nonficFrequency[i][j/2+i-1], minF, maxF, 9, cellWidth * 0.8);
        diameters[i][j+1]= map(ficFrequency[i][j/2+i-1], minF, maxF, 9, cellWidth * 0.8);
        //println(i+" "+j+"  "+diameters[i][j] + " "+ diameters[i][j+1]);
      }
    }
    dummy--;
  }
  //calculate circle's position
  xPositions = new float[years][(years+1)*2];
  dummy = 10;
  for(int j = 0 ; j < (years+1)*2; j+=2){
    for(int i = 0; i < dummy; i++){
      if(j == 0){
        xPositions[i][j] = width/2 - centerMargin/2 - cellWidth/2;
        xPositions[i][j+1] = width/2 + centerMargin/2 + cellWidth/2;
        //println(i + " "+ j + "  "+ xPositions[i][j] + " " + xPositions[i][j+1]);
      }
      else{
        xPositions[i][j] = xPositions[i][j-2] - diameters[i][j-2]/2 - diameters[i][j]/2 - cellWidth/8;
        xPositions[i][j+1] = xPositions[i][j-1] + diameters[i][j+1]/2 + diameters[i][j-1]/2 + cellWidth/8;//diameters[i][j-1];
        //println(i + " "+ j + "  "+ xPositions[i][j] + " " + xPositions[i][j+1]);
      }
    }
    dummy--;
  }
  
  yPositions = new float[years][(years+1)*2];
  dummy = 10;
  for(int j = 0 ; j < (years+1)*2; j+=2){
    for(int i = 0; i < dummy; i++){
      yPositions[i][j] = upperMargin + i * cellWidth ;
      yPositions[i][j+1] = upperMargin + i * cellWidth ;
      //println(i + " "+ j + "  "+ yPositions[i][j] + " " + yPositions[i][j+1]);
    }
    dummy--;
  }
  
  //draw circles
  //first column in the middle
  for(int i = 0; i < years; i++){
    noStroke();
    fill(leftColor(0));
    ellipse(xPositions[i][0], yPositions[i][0], diameters[i][0], diameters[i][0]);
    noStroke();
    fill(rightColor(0));
    ellipse(xPositions[i][1], yPositions[i][1], diameters[i][1], diameters[i][1]);
  }
  
  //other column from outside to center
  for(int i = 0; i < years; i++){
    for(int j=(years-i)*2; j >=0 ; j-=2){
      noStroke();
      fill(leftColor(j));
      ellipse(xPositions[i][j], yPositions[i][j], diameters[i][j], diameters[i][j]);
      noStroke();
      fill(rightColor(j));
      ellipse(xPositions[i][j+1], yPositions[i][j+1], diameters[i][j+1], diameters[i][j+1]);
    }
  }
  
  //Draw Titles
  textAlign(CENTER, CENTER);
  fill(35, 45, 150);
  textFont(hindB, 30);
  text("When “New” Books Are No Longer New", width/2, (upperMargin - 60)/2);
  textFont(hindB, 15);
  text("Annual popularity of new books reflects trend between fiction and non-fiction ", width/2+2, (upperMargin-5)/2);
  //Draw Align
  textFont(hind, 18);
  fill(150);
  for(int i = 0; i < years; i++){
    text(i+2006, width/2, upperMargin* 0.98 + i * cellWidth);
  }
  
  //Draw color bar
  for(int i = 0; i <years; i++){
    noStroke();
    fill(leftColor(i));
    rect(60 ,height -250 + i*20, 20, 20);
    noStroke();
    fill(rightColor(i));
    rect(100 ,height -250 + i*20, 20, 20);
  }
  textAlign(LEFT, CENTER);
  //textFont(hind);
  textSize(11);
  fill(100);
  text("Before Published",140, height -240);
  text("New",140, height -240 + 20);
  text("1 Year After Published",140, height -240 +2*20);
  text("2 Year After Published",140, height -240 +3*20);
  text("3 Year After Published",140, height -240 +4*20);
  text("4 Year After Published",140, height -240 +5*20);
  text("5 Year After Published",140, height -240 +6*20);
  text("6 Year After Published",140, height -240 +7*20);
  text("7 Year After Published",140, height -240 +8*20);
  text("8 Year After Published",140, height -240 +9*20);
  //text("Color Bar", 65, height -240 + 10*20);
  
  //Draw Indicator
  textAlign(CENTER, CENTER);
  textSize(14);
  fill(35, 45, 150);
  text("Fictions", 110, height - 270);
  text("Non-Fictions", 70, height - 290);
  textSize(16);
  fill(100);
  textAlign(LEFT, CENTER);
  text(" Fictions", width/2 + centerMargin/2 + cellWidth/2, 110);
  textAlign(RIGHT, CENTER);
  text("Non-Fictions", width/2 - centerMargin/2 - cellWidth/2, 110);
  
  //Draw size bar
  demoPositionX[0] = width - 380;
  demoDiameter[0] = min2D(diameters);
  space = (max2D(diameters) - min2D(diameters))/7;
  for(int i = 0; i< 8; i++){
    stroke(50);
    strokeWeight(2);
    noFill();
    ellipse(demoPositionX[i], height - 60  - demoDiameter[i]/2, demoDiameter[i], demoDiameter[i]);
    demoDiameter[i+1] = demoDiameter[i] + space;
    demoPositionX[i+1] = demoPositionX[i] + demoDiameter[i]/2 + space + demoDiameter[i];
    if(i ==0 || i ==2 ) continue;
    textAlign(CENTER, CENTER);
    textSize(11);
    fill(100);
    text((int)map(demoDiameter[i], demoDiameter[0], demoDiameter[7], minF, maxF), demoPositionX[i], height - 50);
  }
  textAlign(CENTER, CENTER);
  textSize(11);
  fill(100);
  text(1, demoPositionX[0], height - 85);
  text(23138, demoPositionX[2], height - 95);
  
  //display number in big circles
  if(numDisplay == true){
    textAlign(CENTER, CENTER);
    if(colorMode == 1 || colorMode == 2) fill(255);
    else if(colorMode == 3) fill(0);
    for(int i = 0 ; i < years - 1; i++){
      int a = (int)nonficFrequency[i][i];
      if(a < 30000) textSize(1);
      else if(a < 36000 && a > 30000) textSize(9);
      else if(a < 40000 && a > 36000) textSize(10);
      else if(a < 50000 && a > 40000) textSize(12);
      else if(a < 60000 && a > 50000) textSize(13);
      else if(a < 70000 && a > 60000) textSize(14);
      else textSize(15);
      text((int)nonficFrequency[i][i], xPositions[i][2], yPositions[i][2]-4);
    }
    for(int i = 0 ; i < years - 1; i++){
      int a = (int)nonficFrequency[i][i+1];
      if(a < 30000) textSize(1);
      else if(a < 36000 && a > 30000) textSize(9);
      else if(a < 40000 && a > 36000) textSize(10);
      else if(a < 50000 && a > 40000) textSize(12);
      else if(a < 60000 && a > 50000) textSize(13);
      else if(a < 70000 && a > 60000) textSize(14);
      else textSize(15);
      text((int)nonficFrequency[i][i+1], xPositions[i][4], yPositions[i][4]-4);
    }
    for(int i = 0 ; i < years - 1; i++){
      int a = (int)ficFrequency[i][i];
      if(a < 30000) textSize(1);
      else if(a < 36000 && a > 30000) textSize(9);
      else if(a < 40000 && a > 36000) textSize(10);
      else if(a < 50000 && a > 40000) textSize(12);
      else if(a < 60000 && a > 50000) textSize(13);
      else if(a < 70000 && a > 60000) textSize(14);
      else textSize(15);
      text((int)ficFrequency[i][i], xPositions[i][3], yPositions[i][3]-4);
    }
    for(int i = 0 ; i < years - 1; i++){
      int a = (int)ficFrequency[i][i+1];
      if(a < 30000) textSize(1);
      else if(a < 36000 && a > 30000) textSize(9);
      else if(a < 40000 && a > 36000) textSize(10);
      else if(a < 50000 && a > 40000) textSize(12);
      else if(a < 60000 && a > 50000) textSize(13);
      else if(a < 70000 && a > 60000) textSize(14);
      else textSize(15);
      text((int)ficFrequency[i][i+1], xPositions[i][5], yPositions[i][5]-4);
    }
  }
}